@echo off
SetLocal EnableExtensions EnableDelayedExpansion

:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Copyright (c) 2002-2025 Alexey Kuryakin daqgroup@mail.ru
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Control utility for OPC UA tools.
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:SelfIdentification
set "scriptext=%~x0"
set "scriptfile=%~f0"
set "scriptbase=%~n0"
set "scriptname=%~nx0"
for %%i in ("%~dp0.") do set "scripthome=%%~fi"

:DoSettings
call :SetLang
call :SetOptShare

:ParseCmdLine
set "arg=%~1"
if /i "%arg%" == "menu"                 goto :do_menu
if /i "%arg%" == "install"              goto :do_install
if /i "%arg%" == "uninstall"            goto :do_uninstall
if /i "%arg%" == "opcua-client-gui"     goto :do_opcua_client_gui
if /i "%arg%" == "opcua-modeler-gui"    goto :do_opcua_modeler_gui
if /i "%arg%" == "opcua-spec-dir"       goto :do_opcua_spec_dir
if /i "%arg%" == "opcua-spec-web"       goto :do_opcua_spec_web
if /i "%arg%" == "opcua-articles-dir"   goto :do_opcua_articles_dir
if /i "%arg%" == "opcua-articles-web"   goto :do_opcua_articles_web
if /i "%arg%" == "pylibs-articles-dir"  goto :do_pylibs_articles_dir
if /i "%arg%" == "pylibs-articles-web"  goto :do_pylibs_articles_web
if /i "%arg%" == "pycrwkit-readme-dir"  goto :do_pycrwkit_readme_dir
if /i "%arg%" == "pycrwkit-readme-web"  goto :do_pycrwkit_readme_web
if /i "%arg%" == "asyncua-doc"          goto :do_asyncua_doc
if /i "%arg%" == "opcua-client-example" goto :do_opcua_client_example
if /i "%arg%" == "opcua-server-example" goto :do_opcua_server_example
if /i "%arg%" == "watch-netstat-tcp"    goto :do_watch_netstat_tcp
if /i "%arg%" == "watch-netstat-udp"    goto :do_watch_netstat_udp

call :usage_info
goto :EOF

:::::::::::::::::::::::::::::
:: Set LANG variable to RU/EN
:::::::::::::::::::::::::::::
:SetLang
set "LANG=EN"
if defined CRW_DAQ_SYS_LANG set "LANG=%CRW_DAQ_SYS_LANG:~0,2%"
goto :EOF

::::::::::::::::::::::::::::::::::
:: Set OPTSHARE - shared directory
::::::::::::::::::::::::::::::::::
:SetOptShare
set "OPTSHARE=%SystemDrive%\opt\daqgroup\share"
goto :EOF

::::::::::::::::::::::::::::::::::
:: SetRusEnv <resultVar> <ru> <en>
::::::::::::::::::::::::::::::::::
:SetRusEng
set "%~1=%~3"
if /i "%LANG%" == "RU" set "%~1=%~2"
goto :EOF

::::::::::::::::::::::::::::::
:: strlen <resultVar> <string>
::::::::::::::::::::::::::::::
:strlen
(
 setlocal EnableDelayedExpansion
 set "tmp=%~2"
 if defined tmp (
  set "len=1"
  for %%P in (4096 2048 1024 512 256 128 64 32 16 8 4 2 1) do (
   if "!tmp:~%%P,1!" NEQ "" (
    set /a "len+=%%P"
    set "tmp=!tmp:~%%P!"
   )
  )
 ) ELSE (
  set len=0
 )
)
(
 endlocal
 set "%~1=%len%"
 exit /b
)
goto :EOF

::::::::::::::::::::::::::::::::::::::::
:: menuItem <resultVar> <ru> <en> <fill>
::::::::::::::::::::::::::::::::::::::::
:menuItem
call :SetRusEng item %2 %3
set "fill=%~4" & if "!fill!" == "" set "fill= "
set "fill=%fill:~0,1%"
call :strlen ilen "%item%" & set /a tail=40-!ilen!
if %tail% GTR 0 for /L %%i in (1,1,%tail%) do set "item=!item!%fill%"
set "%~1=%item%"
goto :EOF

:do_menu
set "title=OPCUA Control ..."
call :SetRusEng text1 " ࠢ  OPCUA." "Control panel for  OPCUA."
call :SetRusEng text2 "" "Please"
call :SetRusEng text3 "롨 " "choose the command"
set "text=<big><b><span fgcolor='blue'>%text1%</span>\n%text2% <span fgcolor='green'>%text3%</span>:</b></big>"
call :SetRusEng ColCom "਩" "Comment"
call :SetRusEng ColCmd "" "Command"
set "LANG=EN" & rem Enforce English because problems with Unicode
call :menuItem miOpcuaTools "⨫  OPCUA" "Utilities for OPCUA" "_"
call :menuItem miTestingTools  "⨫  ஢" "Testing Utilities" "_"
call :menuItem miOpcuaClient  "    OPCUA" "Open viewer - OPCUA client GUI"
call :menuItem miOpcuaModeler  " ᭮ ࠭⢠ OPCUA" "OPCUA Address Space GUI"
call :menuItem miDocumentation "㬥  OPCUA" "Documentation on OPCUA" "_"
call :menuItem miOpcuaSpecWeb "䨪樨 OPCUA ()" "Specifications of OPCUA (browse)"
call :menuItem miOpcuaSpecDir "䨪樨 OPCUA ()" "Specifications of OPCUA (folder)"
call :menuItem miOpcuaArticlesWeb "  OPCUA ()" "Articles on OPCUA (browse)"
call :menuItem miOpcuaArticlesDir "  OPCUA ()" "Articles on OPCUA (folder)"
call :menuItem miPylibsArticlesWeb "  Python ()" "Articles on Python (browse)"
call :menuItem miPylibsArticlesDir "  Python ()" "Articles on Python (folder)"
call :menuItem miPycrwkitReadmeWeb " pycrwkit ()" "Modile pycrwkit (browse)"
call :menuItem miPycrwkitReadmeDir " pycrwkit ()" "Module pycrwkit (folder)"
call :menuItem miAsyncuaDoc "㬥  ⥪ Asyncua" "Documentation on library Asyncua"
call :menuItem miOpcuaClientExample " ⮢  opcua" "Launch opcua testing client"
call :menuItem miOpcuaServerExample " ⮢ ࢥ opcua" "Launch opcua testing server"
call :menuItem miWatchNetstatTcp "  祭 TCP" "Watch listening TCP connections"
call :menuItem miWatchNetstatUdp "  祭 UDP" "Watch listening UDP connections"
call :menuItem miExit "   (.. ⬥)" "Exit this menu (same as Cancel)"
call :menuItem miSep1 "_" "_" "_"
call :menuItem miSep2 "_" "_" "_"
unix sleep 1
:menu_loop
set "item=exit"
for /f "usebackq tokens=* delims=" %%i in (`      ^
 unix zenity --title "%title%" --text "%text%"    ^
 --window-icon "%scripthome%\%scriptbase%.png"    ^
 --width 540 --height 620                         ^
 --list --print-column 1                          ^
 --column "%ColCmd%" --column "%ColCom%"          ^
 _____________________ "%miOpcuaTools%"           ^
 opcua-client-gui      "%miOpcuaClient%"          ^
 opcua-modeler-gui     "%miOpcuaModeler%"         ^
 opcua-client-example  "%miOpcuaClientExample%"   ^
 opcua-server-example  "%miOpcuaServerExample%"   ^
 watch-netstat-tcp     "%miWatchNetstatTcp%"      ^
 watch-netstat-udp     "%miWatchNetstatUdp%"      ^ 
 _____________________ "%miDocumentation%"        ^
 opcua-spec-web        "%miOpcuaSpecWeb%"         ^
 opcua-spec-dir        "%miOpcuaSpecDir%"         ^
 opcua-articles-web    "%miOpcuaArticlesWeb%"     ^
 opcua-articles-dir    "%miOpcuaArticlesDir%"     ^
 pylibs-articles-web   "%miPylibsArticlesWeb%"    ^
 pylibs-articles-dir   "%miPylibsArticlesDir%"    ^
 pycrwkit-readme-web   "%miPycrwkitReadmeWeb%"    ^
 pycrwkit-readme-dir   "%miPycrwkitReadmeDir%"    ^
 asyncua-doc           "%miAsyncuaDoc%"           ^
 _____________________ "%miSep1%"                 ^
 exit                  "%miExit%"                 ^
 _____________________ "%miSep2%"                 ^
 `) do set "item=%%~i"
if not "%item%" == "" ( set "item=%item:|= %" & call :set_var item !item! )
call :SetRusEng choose "࠭ " "Choose command"
echo %choose% "%item%"
if /i "%item%" == ""                        goto :EOF
if /i "%item%" == "exit"                    goto :EOF
if /i "%item%" == "cancel"                  goto :EOF
if /i "%item%" == "opcua-client-gui"        goto :do_self_item
if /i "%item%" == "opcua-modeler-gui"       goto :do_self_item
if /i "%item%" == "opcua-spec-dir"          goto :do_self_item
if /i "%item%" == "opcua-spec-web"          goto :do_self_item
if /i "%item%" == "opcua-articles-dir"      goto :do_self_item
if /i "%item%" == "opcua-articles-web"      goto :do_self_item
if /i "%item%" == "pylibs-articles-dir"     goto :do_self_item
if /i "%item%" == "pylibs-articles-web"     goto :do_self_item
if /i "%item%" == "pycrwkit-readme-dir"     goto :do_self_item
if /i "%item%" == "pycrwkit-readme-web"     goto :do_self_item
if /i "%item%" == "asyncua-doc"             goto :do_self_item
if /i "%item%" == "opcua-client-example"    goto :do_self_item
if /i "%item%" == "opcua-server-example"    goto :do_self_item
if /i "%item%" == "watch-netstat-tcp"       goto :do_self_item
if /i "%item%" == "watch-netstat-udp"       goto :do_self_item
if /i "%item:~0,1%" == "_"                  goto :menu_loop
goto :EOF

:usage_info
set "usage=menu|install|uninstall"
set "usage=%usage%|opcua-client-gui|opcua-modeler-gui"
set "usage=%usage%|opcua-spec-dir|opcua-spec-web"
set "usage=%usage%|opcua-articles-dir|opcua-articles-web"
set "usage=%usage%|pylibs-articles-dir|pylibs-articles-web"
set "usage=%usage%|pycrwkit-readme-web|pycrwkit-readme-dir|asyncua-doc"
set "usage=%usage%|opcua-client-example|opcua-server-example"
set "usage=%usage%|watch-netstat-tcp|watch-netstat-udp"
unix printf "Usage: %scriptname% %%s\n" "%usage%"
goto :EOF

:set_var
set "%~1=%~2"
goto :EOF

:execute
echo %*
call %*
exit /b %errorlevel%
goto :EOF

:do_self_item
if defined scriptfile if defined item call "%scriptfile%" "%item%"
goto :EOF

:append_crwkit_path
set "PATH=%PATH%;%SystemDrive%\opt\crwkit\add\exe"
set "PATH=%PATH%;%SystemDrive%\opt\crwdaq\resource\shell"
set "PATH=%PATH%;%SystemDrive%\opt\daqgroup\suite\crwkit\add\exe"
set "PATH=%PATH%;%SystemDrive%\opt\daqgroup\suite\crwdaq\resource\shell"
goto :EOF

:init_daqgroup_menu
set "common_programs=" & set "DaqGroupToolsMenuDir=" & set "opcuacpllnk=" & set "grunexe="
for /f "tokens=* delims=" %%i in ('unix getspecialfolderpath common_programs') do set "common_programs=%%~i"
if not defined common_programs goto :EOF & if not exist "%common_programs%" goto :EOF
set "DaqGroupToolsMenuDir=%common_programs%\DaqGroupTools"
set "opcuacpllnk=%DaqGroupToolsMenuDir%\OpcuaCpl.lnk"
call :append_crwkit_path
set "grunexe=%WinDir%\grun.exe"
set "DIRS=%WinDir%;%SystemDrive%\opt\crwkit\add\exe"
set "DIRS=%DIRS%;%SystemDrive%\opt\crwdaq\resource\shell"
set "DIRS=%WinDir%;%SystemDrive%\opt\daqgroup\suite\crwkit\add\exe"
set "DIRS=%DIRS%;%SystemDrive%\opt\daqgroup\suite\crwdaq\resource\shell"
if not exist "%grunexe%" for %%i in ("grun.exe") do set "grunexe=%%~$DIRS:i"
if not exist "%grunexe%" for %%i in ("grun.exe") do set "grunexe=%%~$PATH:i"
goto :EOF

:xcopy_opcua
set "target=%OPTSHARE%\opcua"
for %%i in ("%scripthome%\..") do set "source=%%~fi"
echo Source "%source%", Target "%target%"
if /i "%source%" == "%target%" goto :EOF
if defined target if exist "%target%" call :execute rmdir /s /q "%target%"
call :execute mkdir "%target%" && call :execute xcopy /s /v /c /i /h /r /y "%source%\*.*" "%target%"
goto :EOF

:do_install
echo do_install:
call :xcopy_opcua
call :init_daqgroup_menu
if not defined opcuacpllnk goto :EOF
if defined DaqGroupToolsMenuDir call :execute mkdir "%DaqGroupToolsMenuDir%"
if defined opcuacpllnk if exist "%opcuacpllnk%" call :execute del /f /q "%opcuacpllnk%"
call :execute crtlnk ^
 --targetpath "%grunexe%" ^
 --arguments "-c -7 %SystemDrive%\opt\daqgroup\share\opcua\scripts\opcua-cpl.cmd menu" ^
 --iconlocation "%scripthome%\%scriptbase%.ico" --showcmd 7 "%opcuacpllnk%" ^
 --workingdirectory "%SystemDrive%\opt\daqgroup\share\opcua\scripts" ^
 --description "Control Panel for OPCUA"
if defined opcuacpllnk if exist "%opcuacpllnk%" call :execute catlnk "%opcuacpllnk%"
goto :EOF

:do_uninstall
echo do_uninstall:
call :init_daqgroup_menu
if not defined opcuacpllnk goto :EOF
if defined opcuacpllnk if exist "%opcuacpllnk%" call :execute del /f /q "%opcuacpllnk%"
if defined DaqGroupToolsMenuDir call :execute rmdir /q "%DaqGroupToolsMenuDir%"
goto :EOF

:do_opcua_client_gui
echo do_opcua_client_gui:
unix grun -h unix pyvenv opcua-client
goto :EOF

:do_opcua_modeler_gui
echo do_opcua_modeler_gui:
unix grun -h unix pyvenv opcua-modeler
goto :EOF

:do_opcua_spec_dir
echo do_opcua_spec_dir:
set "target=%OPTSHARE%\opcua\opcfoundation.org\spec"
unix getapppath explorer.exe firefox.exe chrome.exe -t htmlfile -a .html --run "%target%"
goto :EOF

:do_opcua_spec_web
echo do_opcua_spec_web:
set "target=%OPTSHARE%\opcua\opcfoundation.org\spec"
unix getapppath firefox.exe chrome.exe -t htmlfile -a .html explorer.exe --run "%target%"
goto :EOF

:do_opcua_articles_dir
echo do_opcua_articles_dir:
set "target=%OPTSHARE%\opcua\articles"
unix getapppath explorer.exe firefox.exe chrome.exe -t htmlfile -a .html --run "%target%"
goto :EOF

:do_opcua_articles_web
echo do_opcua_articles_web:
set "target=%OPTSHARE%\opcua\articles"
unix getapppath firefox.exe chrome.exe -t htmlfile -a .html explorer.exe --run "%target%"
goto :EOF

:do_pylibs_articles_dir
echo do_pylibs_articles_dir:
set "target=%OPTSHARE%\pylibs\doc-and-source\articles"
unix getapppath explorer.exe firefox.exe chrome.exe -t htmlfile -a .html --run "%target%"
goto :EOF

:do_pylibs_articles_web
echo do_pylibs_articles_web:
set "target=%OPTSHARE%\pylibs\doc-and-source\articles"
unix getapppath firefox.exe chrome.exe -t htmlfile -a .html explorer.exe --run "%target%"
goto :EOF

:do_pycrwkit_readme_dir
echo do_pycrwkit_readme_dir:
set "target=%OPTSHARE%\pylibs\build_pycrwkit"
unix getapppath explorer.exe firefox.exe chrome.exe -t htmlfile -a .html --run "%target%"
goto :EOF

:do_pycrwkit_readme_web
echo do_pycrwkit_readme_web:
set "target=%OPTSHARE%\pylibs\build_pycrwkit\readme.html"
unix getapppath firefox.exe chrome.exe -t htmlfile -a .html explorer.exe --run "%target%"
goto :EOF

:do_asyncua_doc
echo do_asyncua_doc:
set "target=%OPTSHARE%\pylibs\opcua-workarea\asyncua-doc\asyncua.html"
unix getapppath firefox.exe chrome.exe -t htmlfile -a .html explorer.exe --run "%target%"
goto :EOF

:do_opcua_client_example
echo do_opcua_client_example:
set "pyscript=None"
set "workarea=%OPTSHARE%\pylibs\opcua-workarea"
for /f "tokens=* delims=" %%i in ('dir /s /b %workarea% ^| findstr /c:"examples\client-example.py"') do set "pyscript=%%~fi"
if "%pyscript%" == "None" goto :EOF
for %%i in ("%pyscript%") do set "pyfolder=%%~dpi"
pushd "%pyfolder%" && start "examples\client-example.py" unix pyvenv "%pyscript%"
popd
goto :EOF

:do_opcua_server_example
echo do_opcua_server_example:
set "pyscript=None"
set "workarea=%OPTSHARE%\pylibs\opcua-workarea"
for /f "tokens=* delims=" %%i in ('dir /s /b %workarea% ^| findstr /c:"examples\server-example.py"') do set "pyscript=%%~fi"
if "%pyscript%" == "None" goto :EOF
for %%i in ("%pyscript%") do set "pyfolder=%%~dpi"
pushd "%pyfolder%" && start "examples\server-example.py" unix pyvenv "%pyscript%"
popd
goto :EOF

:do_watch_netstat_tcp
unix cmdfocus /show=1
set command=netstat -n -a -o -p tcp
title %command%
:loop_watch_netstat_tcp
cls & call %command%
ping -n 4 localhost > nul
goto :loop_watch_netstat_tcp
goto :EOF

:do_watch_netstat_udp
unix cmdfocus /show=1
set command=netstat -n -a -o -p udp
title %command%
:loop_watch_netstat_udp
cls & call %command%
ping -n 4 localhost > nul
goto :loop_watch_netstat_udp
goto :EOF
