#!/bin/bash

###########################################################
## Copyright (c) 2002-2025 Alexey Kuryakin daqgroup@mail.ru
###########################################################

###########################################################
## Control utility for OPC UA tools.
###########################################################

###########################################################
source $(crwkit which crwlib_base.sh); # Use base library #
source $(crwkit which crwlib_file.sh); # Use file library #
###########################################################

readonly optshare="/opt/daqgroup/share";

function menuItem(){
 local item="$(langstr ru "$1" en "$2")";
 local fill="$3"; fill="${fill:- }"; fill="${fill:0:1}";
 while [[ ${#item} -lt 40 ]]; do item+="$fill"; done;
 echo "$item$fill";
};

function do_menu(){
 local title="OPCUA Control …";
 local text1="$(langstr ru "Панель управления для" en "Control panel for") OPCUA.";
 local text2="$(langstr ru "Пожалуйста" en "Please")";
 local text3="$(langstr ru "выбирете команду" en "choose the command")";
 local text="<big><b><span fgcolor='blue'>$text1</span>\n\n$text2 <span fgcolor='green'>$text3</span>:</b></big>";
 local ColCom="$(langstr ru "Комментарий" en "Comment")";
 local ColCmd="$(langstr ru "Команда" en "Command")";
 local miOpcuaTools="$(menuItem "Утилиты для OPCUA" "Utilities for OPCUA" "_")";
 local miTestingTools="$(menuItem "Утилиты для тестирования" "Testing Utilities" "_")";
 local miOpcuaClient="$(menuItem "Открыть клиент для обзора OPCUA" "Open viewer - OPCUA client GUI")";
 local miOpcuaModeler="$(menuItem "Редактор адресного пространства OPCUA" "OPCUA Address Space Editor GUI")"; 
 local miDocumentation="$(menuItem "Документация по OPCUA" "Documentation on OPCUA" "_")";
 local miOpcuaSpecWeb="$(menuItem "Спецификации OPCUA (обзор)" "Specifications of OPCUA (browse)")";
 local miOpcuaSpecDir="$(menuItem "Спецификации OPCUA (папка)" "Specifications of OPCUA (folder)")";
 local miOpcuaArticlesWeb="$(menuItem "Статьи по OPCUA (обзор)" "Articles on OPCUA (browse)")";
 local miOpcuaArticlesDir="$(menuItem "Статьи по OPCUA (папка)" "Articles on OPCUA (folder)")";
 local miPylibsArticlesWeb="$(menuItem "Статьи по Python (обзор)" "Articles on Python (browse)")";
 local miPylibsArticlesDir="$(menuItem "Статьи по Python (папка)" "Articles on Python (folder)")";
 local miPycrwkitReadmeWeb="$(menuItem "Модуль pycrwkit (обзор)" "Modile pycrwkit (browse)")";
 local miPycrwkitReadmeDir="$(menuItem "Модуль pycrwkit (папка)" "Module pycrwkit (folder)")";
 local miAsyncuaDoc="$(menuItem "Документация по библиотеке Asyncua" "Documentation on library Asyncua")";
 local miOpcuaClientExample="$(menuItem "Запуск тестового клиента opcua" "Launch opcua testing client")";
 local miOpcuaServerExample="$(menuItem "Запуск тестового сервера opcua" "Launch opcua testing server")";
 local miWatchNetstatTcp="$(menuItem "Обзор слушающих подключений TCP" "Watch listening TCP connections")";
 local miWatchNetstatUdp="$(menuItem "Обзор слушающих подключений UDP" "Watch listening UDP connections")";
 local miExit="$(menuItem "Выйти из меню (т.е. Отменить)" "Exit this menu (same as Cancel)")";
 sleep 1;
 while true; do
  item="$(zenity --title "$title" --text "$text"           \
          --width 540 --height 600                         \
          --window-icon "$scriptHOME/$scriptbase.png"      \
          --list --print-column 1                          \
          --column "$ColCmd" --column "$ColCom"            \
          _____________________ "$miOpcuaTools"            \
          opcua-client-gui      "$miOpcuaClient"           \
          opcua-modeler-gui     "$miOpcuaModeler"          \
          opcua-client-example  "$miOpcuaClientExample"    \
          opcua-server-example  "$miOpcuaServerExample"    \
          watch-netstat-tcp     "$miWatchNetstatTcp"       \
          watch-netstat-udp     "$miWatchNetstatUdp"       \
          _____________________ "$miDocumentation"         \
          opcua-spec-web        "$miOpcuaSpecWeb"          \
          opcua-spec-dir        "$miOpcuaSpecDir"          \
          opcua-articles-web    "$miOpcuaArticlesWeb"      \
          opcua-articles-dir    "$miOpcuaArticlesDir"      \
          pylibs-articles-web   "$miPylibsArticlesWeb"     \
          pylibs-articles-dir   "$miPylibsArticlesDir"     \
          pycrwkit-readme-web   "$miPycrwkitReadmeWeb"     \
          pycrwkit-readme-dir   "$miPycrwkitReadmeDir"     \
          asyncua-doc           "$miAsyncuaDoc"            \
          _____________________ "$(menuItem "_" "_" "_")"  \
          exit                  "$miExit"                  \
          ¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯ "$(menuItem "¯" "¯" "¯")"  \
          2>/dev/null
         )";
  case $item in
   ''|exit|cancel)        return 0; ;;
   opcua-client-gui)      $scriptFILE $item; break; ;;
   opcua-modeler-gui)     $scriptFILE $item; break; ;;
   opcua-spec-dir)        $scriptFILE $item; break; ;;
   opcua-spec-web)        $scriptFILE $item; break; ;;
   opcua-articles-dir)    $scriptFILE $item; break; ;;
   opcua-articles-web)    $scriptFILE $item; break; ;;
   pylibs-articles-dir)   $scriptFILE $item; break; ;;
   pylibs-articles-web)   $scriptFILE $item; break; ;;
   pycrwkit-readme-web)   $scriptFILE $item; break; ;;
   pycrwkit-readme-dir)   $scriptFILE $item; break; ;;
   asyncua-doc)           $scriptFILE $item; break; ;;
   opcua-client-example)  $scriptFILE $item; break; ;;
   opcua-server-example)  $scriptFILE $item; break; ;;
   watch-netstat-tcp)     $scriptFILE $item; break; ;;
   watch-netstat-udp)     $scriptFILE $item; break; ;;
   _*|¯*)                 continue; ;;
   *)                     break; ;;
  esac;
 done;
};

function find_opcua_workarea_file(){
 local wadir="$optshare/pylibs/opcua-workarea";
 local patpath="$1"; local patname="$(basename $patpath)";
 find $wadir -type f -name "$patname" | grep "$patpath" | sort | tail -n1;
};

function do_opcua_client_gui(){
 unix grun pyvenv opcua-client;
};

function do_opcua_modeler_gui(){
 unix grun pyvenv opcua-modeler;
};

function notify_error(){
  local msg="$1"; local hdr="$(date +%Y.%m.%d-%H:%M:%S) - $scriptname";
  unix tooltip-notifier text "$hdr: $msg" preset stdError delay 60000;
};

function do_opcua_spec(){
 local target="$optshare/opcua/opcfoundation.org/spec";
 if [[ -d $target ]]; then
  unix getapppath $* firefox chromium sensible-browser fly-fm xdg-open --run $target;
 else
  notify_error "$(langstr ru "Не найден каталог $target" en "Not found folder $target").";
 fi;
};

function do_opcua_articles(){
 local target="$optshare/opcua/articles";
 if [[ -d $target ]]; then
  unix getapppath $* firefox chromium sensible-browser fly-fm xdg-open --run $target;
 else
  notify_error "$(langstr ru "Не найден каталог $target" en "Not found folder $target").";
 fi;
};

function do_pylibs_articles(){
 local target="$optshare/pylibs/doc-and-source/articles";
 if [[ -d $target ]]; then
  unix getapppath $* firefox chromium sensible-browser fly-fm xdg-open --run $target;
 else
  notify_error "$(langstr ru "Не найден каталог $target" en "Not found folder $target").";
 fi;
};

function do_asyncua_doc(){
 local wanted="asyncua-doc/asyncua.html";
 local target="$(find_opcua_workarea_file $wanted)";
 if [[ -e $target ]]; then
  unix getapppath $* firefox chromium sensible-browser xdg-open --run $target;
 else
  notify_error "$(langstr ru "Не найден файл $wanted" en "Not found file $wanted").";
 fi;
};

function do_pycrwkit_readme(){
 local wanted="build_pycrwkit/readme.html";
 local target="$optshare/pylibs/$wanted";
 if [[ -e $target ]]; then
  unix getapppath $* firefox chromium sensible-browser xdg-open --run $target;
 else
  notify_error "$(langstr ru "Не найден файл $wanted" en "Not found file $wanted").";
 fi;
};

function grun_xterm(){
 unix grun xterm -fa 'PT Mono' -fs 12 -geometry 138x35+50+130 -hold -e "$@";
};

function do_grun_xterm_opcua_script(){
 local wanted="$1";
 local target="$(find_opcua_workarea_file $wanted)";
 if [[ -e $target ]]; then
  if pushd $(dirname $target); then
   grun_xterm unix pyvenv $target;
  else
   notify_error "$(langstr ru "Не могу запустить $wanted" en "Could not start $wanted").";
  fi;
  popd;
 else
  notify_error "$(langstr ru "Не найден файл $wanted" en "Not found file $wanted").";
 fi;
};

function do_watch_netstat_tcp(){
 xterm -fa 'PT Mono' -fs 12 -geometry 120x50+50+50 -e sudo watch netstat -plan4t;
};

function do_watch_netstat_udp(){
 xterm -fa 'PT Mono' -fs 12 -geometry 120x50+50+50 -e sudo watch netstat -plan4u;
};

function do_install(){
 sudo -n cp -fv $scriptHOME/$scriptbase.png     /usr/share/pixmaps/;
 sudo -n cp -fv $scriptHOME/$scriptbase.desktop /usr/share/applications/;
 local png="/usr/share/pixmaps/$scriptbase.png";
 local dsk="/usr/share/applications/$scriptbase.desktop";
 if [[ -e $png ]] && [[ -e $dsk ]]; then
  sudo -n chmod -c ugo+r $png $dsk;
  sudo -n update-desktop-database;
 else
  notify_error "$(langstr ru "Не могу инсталлировать $scriptbase" en "Could not install $scriptbase").";
 fi;
};

function do_uninstall(){
 local png="/usr/share/pixmaps/$scriptbase.png";
 local dsk="/usr/share/applications/$scriptbase.desktop";
 if [[ -e $png ]] || [[ -e $dsk ]]; then
  sudo -n rm -fv $png $dsk;
  sudo -n update-desktop-database;
 fi;
};

function usage_info(){
 local usage="menu|install|uninstall";
 usage+="|opcua-client-gui|opcua-modeler-gui";
 usage+="|opcua-spec-dir|opcua-spec-web";
 usage+="|opcua-articles-dir|opcua-articles-web";
 usage+="|pylibs-articles-dir|pylibs-articles-web";
 usage+="|pycrwkit-readme-web|pycrwkit-readme-dir|asyncua-doc";
 usage+="|opcua-client-example|opcua-server-example";
 usage+="|watch-netstat-tcp|watch-netstat-udp";
 echo "$usage";
};

function opcua_cpl() {
 case $1 in
  menu)                 do_menu; return 0; ;;
  install)              do_install; return; ;;
  uninstall)            do_uninstall; return; ;;
  opcua-client-gui)     do_opcua_client_gui; ;;
  opcua-modeler-gui)    do_opcua_modeler_gui; ;;
  opcua-spec-dir)       do_opcua_spec fly-fm xdg-open; ;;
  opcua-spec-web)       do_opcua_spec firefox chromium; ;;
  opcua-articles-dir)   do_opcua_articles fly-fm xdg-open; ;;
  opcua-articles-web)   do_opcua_articles firefox chromium; ;;
  pylibs-articles-dir)  do_pylibs_articles fly-fm xdg-open; ;;
  pylibs-articles-web)  do_pylibs_articles firefox chromium; ;;
  pycrwkit-readme-dir)  do_pycrwkit_readme fly-fm xdg-open; ;;
  pycrwkit-readme-web)  do_pycrwkit_readme firefox chromium; ;;
  asyncua-doc)          do_asyncua_doc firefox chromium; ;;
  opcua-client-example) do_grun_xterm_opcua_script examples/client-example.py; ;;
  opcua-server-example) do_grun_xterm_opcua_script examples/server-example.py; ;;
  watch-netstat-tcp)    do_watch_netstat_tcp; ;;
  watch-netstat-udp)    do_watch_netstat_udp; ;;
  *)                    echo "${0##*/} $(usage_info)"; ;;
 esac;
}

function main(){
 opcua_cpl "$@";
};

main "$@";

##############
## END OF FILE
##############
